NAME = "roofAI"

ICON = "🏛️"

DESCRIPTION = f"{ICON} everything AI about roofs."

VERSION = "4.91.1"

MARQUEE = "https://github.com/kamangir/assets/blob/main/2023-10-28-16-28-36-88493-predict.gif?raw=true"
